<?php

declare(strict_types=1);

namespace Drupal\debug_headers\Plugin\Block;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Block\Attribute\Block;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Defines a block used to print the current request headers for the page.
 */
#[Block(
  id: "debug_headers",
  admin_label: new TranslatableMarkup('Debug Headers Block'),
  category: new TranslatableMarkup('Debug Headers'),
)]
class DebugHeadersBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The request stack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected RequestStack $requestStack;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected RendererInterface $renderer;

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = new self(
      $configuration,
      $plugin_id,
      $plugin_definition
    );

    $instance->requestStack = $container->get('request_stack');
    $instance->renderer = $container->get('renderer');

    return $instance;
  }

  /**
   * {@inheritDoc}
   */
  public function defaultConfiguration() {
    return [
      'wrap_in_html_comments' => FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account) {
    return AccessResult::allowedIfHasPermission($account, 'view debug headers block');
  }

  /**
   * {@inheritdoc}
   */
  #[\Override]
  public function build(): array {
    if ($this->inPreview) {
      // On the layout management page, generate a preview.
      $render['component'] = [
        '#markup' => '<p>' . $this->t('Debug Header Block Output') . '</p>',
      ];
      return $render;
    }

    $config = $this->configuration;

    // Collect the headers list so that we can print it as a list of items.
    $headersList = [];
    foreach ($this->requestStack->getCurrentRequest()->headers as $header => $value) {
      array_walk($value, function ($value, $key) {
        return Xss::filter($value);
      });
      $item = [
        'value' => ['#markup' => '<strong>' . $header . '</strong>:'],
        'children' => $value,
      ];
      $headersList[] = $item;
    }

    $render['headers'] = [
      '#theme' => 'item_list',
      '#title' => $this->t('Headers List:'),
      '#items' => $headersList,
      '#type' => 'ul',
    ];

    if ($config['wrap_in_html_comments'] === TRUE) {
      // To output the list in HTML comments we need to render it, remove any
      // HTML tags from the string and render it as an inline template.
      $headerList = $this->renderer->renderInIsolation($render);

      $render = [];
      $render['headers'] = [
        '#type' => 'inline_template',
        '#template' => '<!-- {{ headers_list }} -->',
        '#context' => [
          'headers_list' => trim(strip_tags($headerList->__toString())),
        ],
      ];
    }

    return $render;
  }

  /**
   * {@inheritDoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $config = $this->configuration;
    $form = parent::blockForm($form, $form_state);

    $form['wrap_in_html_comments'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Wrap Output In HTML Comments'),
      '#default_value' => $config['wrap_in_html_comments'] ?? FALSE,
      '#description' => $this->t('Use this if you absolutely have to print the output onto the page, but you do not want it to be obviously visible.'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    if (isset($values['wrap_in_html_comments'])) {
      $this->configuration['wrap_in_html_comments'] = (bool) $values['wrap_in_html_comments'];
    }
    parent::blockSubmit($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts(): array {
    // Add some sensible defaults for cache contexts.
    return array_unique(array_merge([
      'url.path',
      'url.query_args',
      'user',
      'user.permissions',
    ], parent::getCacheContexts()));
  }

}
